;; Generated package description from org.el  -*- mode: lisp-data; no-byte-compile: t; lexical-binding:t -*-
(define-package "org" "10.0pre0.20260905.539" "Outline-based notes management and organizer" '((emacs "28.2")) :commit "c8286c97c0843d2ea3a50bc833e17dd9a1919d92" :authors '(("Carsten Dominik" . "carsten.dominik@gmail.com")) :maintainer '("Ihor Radchenko" . "yantar92@posteo.net") :keywords '("outlines" "hypermedia" "calendar" "text") :url "https://orgmode.org")
